@@ -378,4 +378,48 @@ information read from DATA-FILE (as generated by
 (ert-deftest sqlind-ert-pr67 ()
   (sqlind-ert-check-file-syntax "test-data/pr67.sql" "test-data/pr67-syn.eld"))
 
+(ert-deftest sqlind-ert-pr68 ()
+  (sqlind-ert-check-file-syntax "test-data/pr68.sql" "test-data/pr68-syn.eld"))
+
+(ert-deftest sqlind-ert-pr70 ()
+  (sqlind-ert-check-file-syntax "test-data/pr70.sql" "test-data/pr70-syn.eld"))
+
+(ert-deftest sqlind-ert-pr73 ()
+  (sqlind-ert-check-file-syntax "test-data/pr73.sql" "test-data/pr73-syn.eld"))
+
+(ert-deftest sqlind-ert-pr75-postgres ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr75-postgres.sql"
+   "test-data/pr75-postgres-syn.eld"))
+
+(ert-deftest sqlind-ert-pr75-oracle ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr75-oracle.sql"
+   "test-data/pr75-oracle-syn.eld"))
+
+(ert-deftest sqlind-ert-pr80 ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr80.sql"
+   "test-data/pr80-syn.eld"))
+
+(ert-deftest sqlind-ert-pr81 ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr81.sql"
+   "test-data/pr81-syn.eld"))
+
+(ert-deftest sqlind-ert-pr83 ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr83.sql"
+   "test-data/pr83-syn.eld"))
+
+(ert-deftest sqlind-ert-pr84 ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr84.sql"
+   "test-data/pr84-syn.eld"))
+
+(ert-deftest sqlind-ert-pr85 ()
+  (sqlind-ert-check-file-syntax
+   "test-data/pr85.sql"
+   "test-data/pr85-syn.eld"))
+
 ;;; sql-indent-test.el ends here
