@@ -4,7 +4,7 @@
 
 ;; Author: Alex Harsanyi <AlexHarsanyi@gmail.com>
 ;; Created: 27 Sep 2006
-;; Version: 1.3
+;; Version: 1.4
 ;; Keywords: languages sql
 ;; Homepage: https://github.com/alex-hhh/emacs-sql-indent
 ;; Package-Requires: ((cl-lib "0.5"))
@@ -84,7 +84,7 @@ constituents so that syntactic navigation works over them.")
 (defconst sqlind-comment-start-skip "\\(--+\\|/\\*+\\)\\s *"
   "Regexp to match the start of a SQL comment.")
 
-(defconst sqlind-comment-end "\\*+\\/"
+(defconst sqlind-comment-end "\\*+/"
   "Regexp to match the end of a multiline SQL comment.")
 
 (defvar sqlind-comment-prefix "\\*+\\s "
@@ -134,13 +134,13 @@ whitespace, or at the end of the buffer."
 (defun sqlind-search-backward (start regexp limit)
   "Search for REGEXP from START backward until LIMIT.
 Finds a match that is not inside a comment or string, moves point
-to the match and returns it. If no match is found, point is moved
+to the match and returns it.  If no match is found, point is moved
 to LIMIT and nil is returned."
   (goto-char start)
   (let ((done nil))
     (while (and (not done)
                 (re-search-backward regexp limit 'noerror))
-      (unless (sqlind-in-comment-or-string (point))
+      (when (sqlind-same-level-statement (point) start)
         (setq done (point))))
     done))
 
@@ -163,12 +163,13 @@ block label might be empty."
 (defun sqlind-same-level-statement (point start)
   "Return t if POINT is at the same syntactic level as START.
 This means that POINT is at the same nesting level and not inside
-a strinf or comment."
+a string or comment."
   (save-excursion
-    (let ((parse-info (parse-partial-sexp start point)))
-      (not (or (nth 3 parse-info)                 ; inside a string
-               (nth 4 parse-info)                 ; inside a comment
-               (> (nth 0 parse-info) 0))))))      ; inside a nested paren
+    (let ((ppss-point (syntax-ppss point))
+          (ppss-start (syntax-ppss start)))
+      (and (equal (nth 3 ppss-point) (nth 3 ppss-start)) ; string
+           (equal (nth 4 ppss-point) (nth 4 ppss-start)) ; comment
+           (= (nth 0 ppss-point) (nth 0 ppss-start)))))) ; same nesting
 
 (defun sqlind-column-definition-start (pos limit)
   "Find the beginning of a column definition in a select statement.
@@ -251,7 +252,7 @@ symbols and their meaning."
        (save-excursion
          (forward-word 1)
          (sqlind-forward-syntactic-ws)
-         (looking-at "transaction"))))
+         (looking-at "transaction\\>\\|work\\>\\|;"))))
 
 ;;;; Syntactic analysis of SQL code
 
@@ -308,7 +309,7 @@ But don't go before LIMIT."
     (catch 'done
       (while (> (point) (or limit (point-min)))
         (when (re-search-backward
-               ";\\|:=\\|\\_<\\(declare\\|begin\\|cursor\\|for\\|while\\|loop\\|if\\|then\\|else\\|elsif\\)\\_>\\|)"
+               ";\\|:=\\|\\_<\\(declare\\|begin\\|cursor\\|for\\|while\\|loop\\|if\\|then\\|else\\|elsif\\|elseif\\)\\_>\\|)"
                limit 'noerror)
           (unless (sqlind-in-comment-or-string (point))
             (let ((candidate-pos (match-end 0)))
@@ -320,6 +321,9 @@ But don't go before LIMIT."
                     ((looking-at "cursor\\|for\\|while")
                      ;; statement begins at the start of the keyword
                      (throw 'done (point)))
+                    ((looking-at "else?if")
+                     ;; statement begins at the start of the keyword
+                     (throw 'done (point)))
                     ((looking-at "then\\|else")
                      ;; then and else start statements when they are inside
                      ;; blocks, not expressions.
@@ -327,9 +331,6 @@ But don't go before LIMIT."
                      (when (looking-at ";")
                        ;; Statement begins after the keyword
                        (throw 'done candidate-pos)))
-                    ((looking-at "elsif")
-                     ;; statement begins at the start of the keyword
-                     (throw 'done (point)))
                     ((looking-at "if")
                      (when (sqlind-good-if-candidate)
                        ;; statement begins at the start of the keyword
@@ -384,7 +385,7 @@ But don't go before LIMIT."
 ;;;;; Find the syntax and beginning of the current block
 
 (defconst sqlind-end-statement-regexp
-  "\\_<end\\_>\\(?:[ \t\n\r\t]*\\)\\(if\\_>\\|loop\\_>\\|case\\_>\\)?\\(?:[ \t\n\r\f]*\\)\\([a-z0-9_]+\\)?"
+  "\\_<end\\_>\\(?:[ \t\n\r\f]*\\)\\(if\\_>\\|loop\\_>\\|case\\_>\\)?\\(?:[ \t\n\r\f]*\\)\\([a-z0-9_]+\\)?"
   "Match an end of statement.
 Matches a string like \"end if|loop|case MAYBE-LABEL\".")
 
@@ -478,7 +479,7 @@ See also `sqlind-beginning-of-block'"
            nil))))))
 
 (defun sqlind-good-if-candidate ()
-  "Return true if point is on an actual if statement.
+  "Return non-nil if point is on an actual if statement.
 We try to avoid false positives, like \"end if\" or the various
 \"drop STUFF if exists\" variants."
   (and (looking-at "if")
@@ -487,7 +488,7 @@ We try to avoid false positives, like \"end if\" or the various
          (forward-word -1)
          ;; we don't want to match an "end if", and things like "drop index if
          ;; exists..." and "create index if not exist..."
-         (not (looking-at "end\\|table\\|view\\|index\\|trigger\\procedude\\|function\\|package\\|body")))))
+         (not (looking-at "end\\|table\\|view\\|index\\|trigger\\|procedure\\|function\\|package\\|body")))))
 
 (defun sqlind-maybe-if-statement ()
   "If (point) is on an IF statement, report its syntax."
@@ -503,7 +504,7 @@ We try to avoid false positives, like \"end if\" or the various
                        "bad closing for if block" (point) pos))))))))
 
 (defun sqlind-maybe-case-statement ()
-  "If (point) is on a case statement"
+  "If (point) is on a case statement."
   (when (looking-at "case")
     (save-excursion
       (sqlind-backward-syntactic-ws)
@@ -639,7 +640,7 @@ See also `sqlind-beginning-of-block'"
 	  'declare-statement
         (list 'syntax-error "nested declare block" (point) (point))))))
 
-(defun sqlind-maybe-skip-mysql-create-options ()
+(defun sqlind-maybe-skip-create-options ()
   "Move point past any MySQL option declarations.
 
 Statements like \"CREATE VIEW\" or \"CREATE TABLE\" can have
@@ -647,30 +648,78 @@ various options betwen the CREATE keyword and the thing being
 created.  If such options exist at (point) the cursor is moved
 past them.
 
-Currently we move over the following options:
+Currently we move over the following options, for different
+products:
+
+MySQL:
 
   TEMPORARY
   ALGORITHM = {UNDEFINED | MERGE | TEMPTABLE}
   DEFINER = { user | CURENT_USER }
   SQL SECURITY { DEFINER | INVOKER }
 
+PostgresSQL
+
+  TEMP
+  TEMPORARY
+  GLOBAL
+  LOCAL
+  UNLOGGED
+  MATERIALIZED
+
+Oracle
+
+  PRIVATE
+  TEMP
+  TEMPORARY
+  MATERIALIZED
+
 We don't consider if the options are valid or not for the thing
 being created.  We just skip any and all of them that are
 present."
-  (when (eq sql-product 'mysql)
-    (catch 'finished
-      (while t
-        (cond
-          ((looking-at "temporary\\_>")
-           (goto-char (match-end 0))
-           (sqlind-forward-syntactic-ws))
-          ((looking-at "\\(definer\\|algorithm\\)\\(\\s-\\|[\n]\\)*=\\(\\s-\\|[\n]\\)*\\S-+")
-           (goto-char (match-end 0))
-           (sqlind-forward-syntactic-ws))
-          ((looking-at "sql\\(\\s-\\|[\n]\\)+security\\(\\s-\\|[\n]\\)+\\S-+")
-           (goto-char (match-end 0))
-           (sqlind-forward-syntactic-ws))
-          (t (throw 'finished nil)))))))
+  (cond
+    ((eq sql-product 'mysql)
+     (catch 'finished
+       (while t
+         (cond
+           ((looking-at "temporary\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "\\(definer\\|algorithm\\)\\(\\s-\\|[\n]\\)*=\\(\\s-\\|[\n]\\)*\\S-+")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "sql\\(\\s-\\|[\n]\\)+security\\(\\s-\\|[\n]\\)+\\S-+")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           (t (throw 'finished nil))))))
+    ((eq sql-product 'postgres)
+     (catch 'finished
+       (while t
+         (cond
+           ((looking-at "temp\\(orary\\)?\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "\\(global\\|local\\|unlogged\\)\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "materialized\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           (t (throw 'finished nil))))))
+    ((eq sql-product 'oracle)
+     (catch 'finished
+       (while t
+         (cond
+           ((looking-at "temp\\(orary\\)\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "materialized\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           ((looking-at "private\\_>")
+            (goto-char (match-end 0))
+            (sqlind-forward-syntactic-ws))
+           (t (throw 'finished nil))))))))
 
 (defun sqlind-maybe-create-statement ()
   "If (point) is on a CREATE statement, report its syntax.
@@ -682,7 +731,7 @@ See also `sqlind-beginning-of-block'"
 	;; let's see what are we creating
 	(goto-char (match-end 0))
         (sqlind-forward-syntactic-ws)
-        (sqlind-maybe-skip-mysql-create-options)
+        (sqlind-maybe-skip-create-options)
 	(let ((what (intern (downcase (buffer-substring-no-properties
 				       (point)
 				       (progn (forward-word) (point))))))
@@ -754,6 +803,14 @@ See also `sqlind-beginning-of-block'"
 	      ;; not a procedure after all.
 	      (throw 'exit nil)))
 
+          ;; Find out if it is a drop procedure or function statement
+          (save-excursion
+            (sqlind-backward-syntactic-ws)
+            (forward-word -1)
+            (when (looking-at "drop")
+              ;; not a procedure after all
+              (throw 'exit nil)))
+
 	  ;; so it is a definition
 
 	  ;; if the procedure starts with "create or replace", move
@@ -829,7 +886,7 @@ See also `sqlind-beginning-of-block'"
 
 (defconst sqlind-start-block-regexp
   (concat "\\(\\_<"
-	  (regexp-opt '("if" "then" "else" "elsif" "loop"
+	  (regexp-opt '("if" "then" "else" "elsif" "elseif" "loop"
 			"begin" "declare" "create" "alter" "exception"
 			"procedure" "function" "end" "case") t)
 	  "\\_>\\)\\|)\\|\\$\\$")
@@ -947,7 +1004,7 @@ reverse order (a stack) and is used to skip over nested blocks."
 (defconst sqlind-select-clauses-regexp
   (concat
    "\\_<\\("
-   "\\(union\\|intersect\\|minus\\)?[ \t\r\n\f]*select\\|"
+   "\\(\\(union\\(\\s-+all\\)?\\)\\|intersect\\|minus\\|except\\)?[ \t\r\n\f]*select\\|"
    "\\(bulk[ \t\r\n\f]+collect[ \t\r\n\f]+\\)?into\\|"
    "from\\|"
    "where\\|"
@@ -960,7 +1017,36 @@ reverse order (a stack) and is used to skip over nested blocks."
    "\\)\\_>"))
 
 (defconst sqlind-select-join-regexp
-  (regexp-opt '("inner" "left" "right" "natural" "cross") 'symbols))
+  (regexp-opt '("inner" "left" "right" "natural" "cross" "full") 'symbols))
+
+(defconst sqlind-join-condition-regexp
+  (regexp-opt '("on" "using" "and" "or") 'symbols))
+
+(defun sqlind-find-join-start (start limit)
+  "Look backwards for the start of a JOIN statement,
+begin looking for it at the START position in the buffer, and
+look backwards until LIMIT is reached.  Returns the buffer
+position where the JOIN condition starts, or nil if no JOIN
+statement is found."
+  (save-excursion
+    (when (sqlind-search-backward start "\\bjoin\\b" limit)
+      (let ((candidate (point)))
+        (forward-char -1)
+        (sqlind-backward-syntactic-ws)
+        (backward-word)
+        (if (looking-at sqlind-select-join-regexp)
+            (point)
+          ;; The "outer" keyword is composed with a "left" or "right" keyword
+          ;; so we need to move backwards one more word.
+          (if (looking-at "\\bouter\\b")
+              (progn
+                (forward-char -1)
+                (sqlind-backward-syntactic-ws)
+                (backward-word)
+                (if (looking-at "\\b\\(left\\|right\\)\\b")
+                    (point)
+                  candidate))
+            candidate))))))
 
 (defun sqlind-syntax-in-select (pos start)
   "Return the syntax ar POS which is inside a \"select\" statement at START."
@@ -976,7 +1062,7 @@ reverse order (a stack) and is used to skip over nested blocks."
       ;; when we are not looking at a select component, find the
       ;; nearest one from us.
 
-      (while (re-search-backward sqlind-select-clauses-regexp start t)
+      (while (sqlind-search-backward (point) sqlind-select-clauses-regexp start)
 	(let* ((match-pos (match-beginning 0))
 	       (clause (sqlind-match-string 0)))
 	  (setq clause (replace-regexp-in-string "[ \t\r\n\f]" " " clause))
@@ -1002,22 +1088,16 @@ reverse order (a stack) and is used to skip over nested blocks."
 	       (forward-char -1)
 	       (sqlind-backward-syntactic-ws)
 	       (unless (looking-at ",")
-		 ;; yep, we are in the from section.
-		 ;; if this line starts with 'on' or the previous line
-		 ;; ends with 'on' we have a join condition
+		 ;; We are in the from section.  If this line starts with 'on'
+		 ;; or the previous line ends with 'on' we have a join
+		 ;; condition
 		 (goto-char pos)
-		 (when (or (looking-at "on")
-			   (progn (forward-word -1) (looking-at "on")))
+		 (when (or (looking-at sqlind-join-condition-regexp)
+			   (progn (forward-word -1) (looking-at sqlind-select-join-regexp)))
 		   ;; look for the join start, that will be the anchor
-                   (when (sqlind-search-backward (point) "\\bjoin\\b" start)
-                     (let ((candidate (point)))
-                       (forward-char -1)
-                       (sqlind-backward-syntactic-ws)
-                       (backward-word)
-                       (throw 'finished
-                         (if (looking-at sqlind-select-join-regexp)
-                             (cons 'select-join-condition (point))
-                           (cons 'select-join-condition candidate)))))))
+                   (let ((jstart (sqlind-find-join-start (point) start)))
+                     (when jstart
+                       (throw 'finished (cons 'select-join-condition jstart))))))
 
 	       ;; if this line starts with a ',' or the previous line starts
 	       ;; with a ',', we have a new table
@@ -1250,7 +1330,7 @@ KIND is the symbol determining the type of the block ('if, 'loop,
 			  (list 'block-end start-kind start-label)))
 		   anchor))))
 
-	     ((memq block-kind '(else elsif))
+	     ((memq block-kind '(else elsif elseif))
 	      ;; search the enclosing then context and refine form there.  The
 	      ;; `cdr' in sqlind-syntax-of-line is used to remove the
 	      ;; block-start context for the else clause
@@ -1411,13 +1491,24 @@ not a statement-continuation POS is the same as the
                  (when (looking-at "case")
                    (push (sqlind-syntax-in-case pos (point)) context))))))
 
+         (when (eq (sqlind-syntax-symbol context) 'nested-statement-continuation)
+           (save-excursion
+             ;; Look for a join expression inside a nested statement, see #70
+	     (goto-char pos)
+	     (when (or (looking-at sqlind-join-condition-regexp)
+		       (progn (forward-word -1) (looking-at sqlind-join-condition-regexp)))
+	       ;; look for the join start, that will be the anchor
+               (let ((jstart (sqlind-find-join-start (point) anchor)))
+                 (when jstart
+                   (push (cons 'select-join-condition jstart) context))))))
+
          ))
 
     ;; create block start syntax if needed
 
     ((and (eq syntax-symbol 'in-block)
-          (memq (nth 1 syntax) '(if elsif then case))
-          (looking-at "\\(then\\|\\(els\\(e\\|if\\)\\)\\)\\_>"))
+          (memq (nth 1 syntax) '(if elsif elseif then case))
+          (looking-at "\\(then\\|\\(els\\(e\\|e?if\\)\\)\\)\\_>"))
      (let ((what (intern (sqlind-match-string 0))))
        ;; the only invalid combination is a then statement in
        ;; an (in-block "then") context
@@ -1587,16 +1678,16 @@ The following syntax symbols are defined for SQL code:
   are buffer locations denoting the problematic region.  ANCHOR
   is undefined for this syntax info
 
-- in-comment -- line is inside a multi line comment, ANCHOR is
+- `in-comment' -- line is inside a multi line comment, ANCHOR is
   the start of the comment.
 
-- comment-start -- line starts with a comment.  ANCHOR is the
+- `comment-start' -- line starts with a comment.  ANCHOR is the
   start of the enclosing block.
 
-- in-string -- line is inside a string, ANCHOR denotes the start
+- `in-string' -- line is inside a string, ANCHOR denotes the start
   of the string.
 
-- toplevel -- line is at toplevel (not inside any programming
+- `toplevel' -- line is at toplevel (not inside any programming
   construct).  ANCHOR is usually (point-min).
 
 - (in-block BLOCK-KIND LABEL) -- line is inside a block
@@ -1709,7 +1800,7 @@ clause (select, from, where, etc) in which the current point is.
   table was defined and a new one is about to start.
 
 - select-table-continuation -- line is inside the from clause,
-  inside a table definition which starts on a previous line. Note
+  inside a table definition which starts on a previous line.  Note
   that ANCHOR still points to the start of the select statement
   itself.
 
@@ -2159,7 +2250,11 @@ it will indent lines starting with JOIN keywords to line up with
 the FROM keyword."
   (save-excursion
     (back-to-indentation)
-    (if (looking-at "\\b\\(\\(inner\\|outer\\|cross\\)\\s-+\\)?join\\b")
+    ;; NOTE: we are a bit loose here as we only look for the first keyword
+    ;; which might indicate a join regexp, e.g. we are happy to see "left"
+    ;; even though, the correct statement is "left outer? join"
+    (if (or (looking-at sqlind-select-join-regexp)
+            (looking-at "\\bjoin\\b"))
         (sqlind-lineup-to-anchor syntax base-indentation)
       base-indentation)))
 
@@ -2196,7 +2291,7 @@ statement.  For example:
   )
 
 This function only makes sense in a
-'nested-statement-continuation sytnax indentation rule."
+'nested-statement-continuation SYTNAX indentation rule."
   (save-excursion
     (goto-char (sqlind-anchor-point syntax))
     (forward-char 1)
